import {ModelComponentCreator} from "./ModelComponentCreator";
import {GameModel} from "../../GameModel";
import {ModelComponent} from "../../ModelComponent";
import {TemplateTypeUtilities} from "../../templates/TemplateTypeUtilities";
import {Gamesystem} from "../../gamesystems/Gamesystem";
import {SimpleGamesystem} from "../../gamesystems/SimpleGamesystem";
import {ProductGamesystem} from "../../gamesystems/ProductGamesystem";
import {SimpleTemplateGamesystem} from "../../templates/simpleGamesystem/SimpleTemplateGamesystem";
import {ProductTemplateCreator} from "../../templates/productGamesystem/ProductTemplateCreator";

export class GamesystemCreator extends ModelComponentCreator{


  constructor(context: string, gameModel: GameModel, selectedComponent: ModelComponent | undefined) {
    super(context, gameModel, selectedComponent);
  }

  createModelComponent(): ModelComponent | undefined {
    const templateType = TemplateTypeUtilities.fromString(this.context);
    let simpleGamesystem;
    if(templateType == undefined) /**Aka normal**/{
      simpleGamesystem = new SimpleGamesystem("New Simple Gamesystem", "");
    } else {
      simpleGamesystem = new SimpleTemplateGamesystem("New Simple Gamesystem", "", templateType)
    }

    if(this.selectedComponent !== undefined && this.selectedComponent instanceof Gamesystem)/**Aka productGamesystem**/ {
      let productParentsystem: ProductGamesystem;
      if(this.selectedComponent instanceof SimpleTemplateGamesystem) {
        productParentsystem = ProductTemplateCreator.constructTemplateFromSimpleGamesystem(this.selectedComponent, this.gameModel!, this.selectedComponent.templateType);
      } else if(this.selectedComponent instanceof SimpleGamesystem) {
        productParentsystem = ProductGamesystem.constructFromSimpleGamesystem(this.selectedComponent, this.gameModel);
      } else {
        productParentsystem = this.selectedComponent as ProductGamesystem;
      }

      productParentsystem.addChildGamesystem(simpleGamesystem);
    } else {
      this.gameModel!.gamesystems.push(simpleGamesystem);
    }

    return simpleGamesystem;
  }


}
